/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.junit.Assert;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.junit.Configuration;

/**
 * @author Marc Jambert - EBM WebSourcing
 */
public final class ServiceConfiguration extends Configuration {

    public enum ServiceType {
        PROVIDE, CONSUME;
    }
    
    private final Map<QName, String> servicesSectionParameters;

    private final List<ServiceConfiguration> serviceConfigurationDependencies;

    private final ServiceType serviceType;

    private final QName interfaceQName;

    private final QName serviceQName;

    /**
     * It is not final because if it is an autogenerate endpoint, it will be changed
     */
    private String endpointName;

    private final URL serviceDescription;

    private String operation;

    private long timeout;

    private MEPType mep;
    
    /**
     * 
     * @param configurationName
     * @param interfaceQName
     *            Interface name of the service configuration. Cannot be <code>null</code>
     * @param serviceQName
     *            Service name of the service configuration. Cannot be <code>null</code>
     * @param endpointName
     *            Endpoint name of the service configuration. Cannot be <code>null</code>
     * @param serviceType
     * @param serviceDescription
     */
    public ServiceConfiguration(final QName interfaceQName, final QName serviceQName,
            final String endpointName, final ServiceType serviceType, final URL serviceDescription) {
        assert interfaceQName != null;
        assert serviceQName != null;
        assert endpointName != null;
        assert serviceType != null;
		
        this.interfaceQName = interfaceQName;
        this.serviceQName = serviceQName;
        this.endpointName = endpointName;
        this.serviceType = serviceType;
        this.serviceDescription = serviceDescription;
        this.serviceConfigurationDependencies = new ArrayList<ServiceConfiguration>();
        this.servicesSectionParameters = new HashMap<QName, String>();
    }

    public ServiceConfiguration(final QName interfaceName, final QName serviceName,
            final String endpointName, final ServiceType serviceType) {
        this(interfaceName, serviceName, endpointName, serviceType, null);
    }
    
    public ServiceType getServiceType() {
        return serviceType;
    }

    public final String getEndpointName() {
        return endpointName;
    }

    public URL getServiceDescription() {
        return serviceDescription;
    }

    public QName getInterfaceQName() {
        return interfaceQName;
    }

    public QName getServiceQName() {
        return serviceQName;
    }
    
    public String getOperation() {
        return operation;
    }
    
    public long getTimeout() {
        return timeout;
    }

    public MEPType getMEP() {
        return mep;
    }

    /**
     * @param operation
     *            CDK operation. Will have the same NS as the service. Can't be <code>null</code>.
     */
    public void setOperation(final String operation) {
        assert operation != null;
        this.operation = operation;
    }
    
    /**
     * 
     * @param timeout
     *            CDK timeout
     */
    public void setTimeout(final long timeout) {
        this.timeout = timeout;
    }

    /**
     * 
     * @param mep CDK MEP. Can't be null
     */
    public void setMEP(final MEPType mep) {
        assert mep != null;
        this.mep = mep;
    }

    public void addServiceConfigurationDependency(final ServiceConfiguration serviceConfiguration) {
        assert serviceConfiguration != null;
        this.serviceConfigurationDependencies.add(serviceConfiguration);
    }

    public ServiceConfiguration getServiceConfigurationDependency(final int index) {
        return this.serviceConfigurationDependencies.get(index);
    }

    /**
     * @return a copy of service dependencies {@link List}
     */
    public ServiceConfiguration[] getServiceConfigurationDependencies() {
        return this.serviceConfigurationDependencies.toArray(new ServiceConfiguration[this.serviceConfigurationDependencies
                .size()]);
    }

    /**
     * Parameters for the services section of the SU.
     * 
     * It is NOT used on {@link ServiceConfiguration} of type {@link ServiceType#CONSUME}.
     * 
     * @return
     */
    public final QName[] getServicesSectionParameterNames() {
        return servicesSectionParameters.keySet().toArray(new QName[servicesSectionParameters.size()]);
    }

    public final boolean hasServicesSectionParameter(final QName parameterName) {
        assert parameterName != null;
        return servicesSectionParameters.containsKey(parameterName);
    }

    public final String getServicesSectionParameter(final QName parameterName) {
        assert parameterName != null;
        return servicesSectionParameters.get(parameterName);
    }

    public final void setServicesSectionParameter(final QName parameterName, final String parameterValue) {
        assert parameterName != null;
        assert parameterValue != null;
        servicesSectionParameters.put(parameterName, parameterValue);
    }

    @Override
    public String toString() {
        return "ServiceConfiguration [serviceType=" + serviceType + ", interfaceQName=" + interfaceQName
                + ", serviceQName=" + serviceQName + ", endpointName=" + endpointName + ", serviceDescription="
                + serviceDescription + "]";
    }

    /**
     * Package visibility because it is only meant to be used by {@link ComponentInstanceImpl} when it is an
     * autogenerate endpoint.
     * 
     * @param endpointName2
     */
    void setEndpointName(String endpointName) {
        Assert.assertEquals(this.endpointName, "autogenerate");
        this.endpointName = endpointName;
    }

}
