/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.mock;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.mockito.Mockito;
import org.ow2.petals.component.framework.junit.Component;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;
import org.ow2.petals.jbi.messaging.exchange.impl.DefaultMessageExchangeFactory;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;

/**
 * 
 * One of the responsibility of this class is to present to the tested component {@link MessageExchange} that are
 * {@link MessageExchangeWrapper}.
 * 
 * In particular it means that {@link #accept()} and {@link #accept(long)} returns {@link MessageExchangeWrapper} to the
 * tested component and {@link #send(MessageExchange)}, {@link #sendSync(MessageExchange)} and
 * {@link #sendSync(MessageExchange, long)} take {@link MessageExchangeWrapper} as parameters.
 * 
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class MockDeliveryChannel implements DeliveryChannel {

    private final Logger logger;

    /**
     * For messages from the testing client or other (non-testing client) services to the tested component
     */
    private final BlockingQueue<PetalsMessageExchange> incomingExchangesQueue;

    /**
     * For messages from the tested component to the testing client
     */
    private final BlockingQueue<PetalsMessageExchange> exchangesToClientQueue;

    /**
     * For messages from the tested component to other (non-testing client) services
     */
    private final BlockingQueue<PetalsMessageExchange> exchangesToServerQueue;

    private String synchronousExchangeId;

    private final Semaphore synchronousExchangeSemaphore = new Semaphore(0);

    public MockDeliveryChannel(final Logger logger) {
        this.logger = logger;
        this.incomingExchangesQueue = new LinkedBlockingQueue<PetalsMessageExchange>();
        this.exchangesToClientQueue = new LinkedBlockingQueue<PetalsMessageExchange>();
        this.exchangesToServerQueue = new LinkedBlockingQueue<PetalsMessageExchange>();
    }

    public void enqueueIncomingExchange(final PetalsMessageExchange msgExchange) {
        try {
            this.incomingExchangesQueue.put(msgExchange);
        } catch (final InterruptedException e) {
            throw new UncheckedException(e);
        }
    }

    public void enqueueExchangeToServer(final PetalsMessageExchange msgExchange) {
        try {
            this.exchangesToServerQueue.put(msgExchange);
        } catch (final InterruptedException e) {
            throw new UncheckedException(e);
        }
    }

    public void enqueueExchangeToClient(final PetalsMessageExchange msgExchange) {
        try {
            this.exchangesToClientQueue.put(msgExchange);
        } catch (final InterruptedException e) {
            throw new UncheckedException(e);
        }
    }

    /**
     * @param timeout
     *            Timeout in millisecond to dequeue the message exchange. If <code>0</code>, no timeout will occur.
     * @return
     */
    public PetalsMessageExchange dequeueIncomingExchange(final long timeout) {
        try {
            if (timeout == 0) {
                return this.incomingExchangesQueue.take();
            } else {
                return this.incomingExchangesQueue.poll(timeout, TimeUnit.MILLISECONDS);
            }
        } catch (final InterruptedException e) {
            throw new UncheckedException(e);
        }
    }

    public void clearIncomingExchanges() {
        this.incomingExchangesQueue.clear();
    }

    /**
     * @param timeout
     *            Timeout in millisecond to dequeue the message exchange. If <code>0</code>, no timeout will occur.
     * @return
     */
    public PetalsMessageExchange dequeueExchangeToServer(final long timeout) {
        try {
            if (timeout == 0) {
                return this.exchangesToServerQueue.take();
            } else {
                return this.exchangesToServerQueue.poll(timeout, TimeUnit.MILLISECONDS);
            }
        } catch (final InterruptedException e) {
            throw new UncheckedException(e);
        }
    }

    public void clearExchangesToServer() {
        this.exchangesToServerQueue.clear();
    }

    public int getExchangesToServerCount() {
        return this.exchangesToServerQueue.size();
    }

    /**
     * @param timeout
     *            Timeout in millisecond to dequeue the message exchange. If <code>0</code>, no timeout will occur.
     * @return
     */
    public PetalsMessageExchange dequeueExchangeToClient(final long timeout) {
        try {
            if (timeout == 0) {
                return this.exchangesToClientQueue.take();
            } else {
                return this.exchangesToClientQueue.poll(timeout, TimeUnit.MILLISECONDS);
            }
        } catch (final InterruptedException e) {
            throw new UncheckedException(e);
        }
    }

    public void clearExchangesToClient() {
        this.exchangesToClientQueue.clear();
    }

    public int getExchangesToClientCount() {
        return this.exchangesToClientQueue.size();
    }

    @Override
    public MessageExchange accept() throws MessagingException {
        return this.accept(0);
    }

    @Override
    public MessageExchange accept(final long timeoutMS) throws MessagingException {
        try {
            final PetalsMessageExchange exchange;
            if (timeoutMS == 0) {
                exchange = this.incomingExchangesQueue.take();
            } else {
                exchange = this.incomingExchangesQueue.poll(timeoutMS, TimeUnit.MILLISECONDS);
            }
            if (exchange.getExchangeId().equals(this.synchronousExchangeId)) {
                this.synchronousExchangeSemaphore.release();
                return accept();
            } else {
                return new MessageExchangeWrapper(exchange);
            }
        } catch (final InterruptedException e) {
            throw new MessagingException(e);
        }
    }

    @Override
    public void close() throws MessagingException {
        // TODO Auto-generated method stub
    }

    @Override
    public MessageExchangeFactory createExchangeFactory() {
        final MessageExchangeFactory exchangeFactoryImpl = DefaultMessageExchangeFactory
                .create(Mockito.mock(ServiceEndpoint.class),
                        this.logger);
        return exchangeFactoryImpl;
    }

    @Override
    public MessageExchangeFactory createExchangeFactory(final QName interfaceName) {
        final MessageExchangeFactory exchangeFactoryImpl = DefaultMessageExchangeFactory
                .createForInterface(
                        Mockito.mock(ServiceEndpoint.class), this.logger, interfaceName);
        return exchangeFactoryImpl;
    }

    @Override
    public MessageExchangeFactory createExchangeFactory(final ServiceEndpoint endpoint) {
        final MessageExchangeFactory exchangeFactoryImpl = DefaultMessageExchangeFactory
                .createForEndpoint(
                        Mockito.mock(ServiceEndpoint.class), this.logger, endpoint);
        return exchangeFactoryImpl;
    }

    @Override
    public MessageExchangeFactory createExchangeFactoryForService(final QName serviceName) {
        final MessageExchangeFactory exchangeFactoryImpl = DefaultMessageExchangeFactory
                .createForService(
                        Mockito.mock(ServiceEndpoint.class), this.logger, serviceName);
        return exchangeFactoryImpl;
    }

    @Override
    public void send(final MessageExchange messageExchange) throws MessagingException {
        if (messageExchange.getEndpoint() == null) {
            this.resolveEndpoint(messageExchange);
        }

        final MessageExchangeWrapper exchange = (MessageExchangeWrapper) messageExchange;

        if (messageExchange.getRole().equals(Role.PROVIDER)) {
            exchange.getMessageExchange().setRole(Role.CONSUMER);
            this.enqueueExchangeToClient(exchange.getMessageExchange());
        } else {
            exchange.getMessageExchange().setRole(Role.PROVIDER);
            this.enqueueExchangeToServer(exchange.getMessageExchange());
        }
    }

    private void resolveEndpoint(final MessageExchange messageExchange) throws MessagingException {
        final ServiceEndpoint endpoint = MockComponentContext.resolveServiceEndpoint(messageExchange
                .getService());
        if (endpoint != null) {
            messageExchange.setEndpoint(endpoint);
        } else {
            throw new MessagingException("Endpoint not avalaible");
        }
    }

    @Override
    public boolean sendSync(final MessageExchange msgExchange) throws MessagingException {
        return this.sendSync(msgExchange, 0);
    }

    @Override
    public boolean sendSync(final MessageExchange msgExchange, final long timeoutMS)
            throws MessagingException {
        boolean returnValue = false;
        this.synchronousExchangeId = msgExchange.getExchangeId();
        msgExchange.setProperty(Component.SENDSYNC_EXCHANGE_PROPERTY, true);
        this.send(msgExchange);
        try {
            if (timeoutMS > 0) {
                returnValue = this.synchronousExchangeSemaphore.tryAcquire(timeoutMS,
                        TimeUnit.MILLISECONDS);

            } else {
                this.synchronousExchangeSemaphore.acquire();
                returnValue = true;
            }
        } catch (final InterruptedException e) {
            throw new UncheckedException(e);
        }
        return returnValue;
    }
}
